<?php

require_once 'Horde/History.php';

/**
 * Genie_Driver:: defines an API for implementing storage backends for Genie.
 *
 * $Horde: genie/lib/Driver.php,v 1.8 2004/04/07 14:43:15 chuck Exp $
 *
 * @author  Jon Parise <jon@horde.org>
 * @version $Revision: 1.8 $
 * @since   Genie 0.1
 * @package Genie
 */
class Genie_Driver {

    /**
     * Array holding the current wishlist. Each array entry is a hash
     * describing a item. The array is indexed numerically by item ID.
     * @var array $_items
     */
    var $_items = array();

    /**
     * The current wishlist.
     * @var string $_wishlist
     */
    var $_wishlist = '';

    /**
     * Hash containing connection parameters.
     * @var array $_params
     */
    var $_params = array();

    /**
     * Lists items based on the given criteria.  All items will be returned
     * by default.
     *
     * @return array    Returns a list of the requested items.
     */
    function listItems()
    {
        return $this->_items;
    }

    /**
     * Attempts to return a concrete Genie_Driver instance based on $driver.
     *
     * @param string    $driver     The type of concrete Genie_Driver subclass
     *                              to return. The is based on the storage
     *                              driver ($driver). The code is dynamically
     *                              included.
     *
     * @param string    $wishlist   The wishlist to load.
     *
     * @param array     $params     (optional) A hash containing any additional
     *                              configuration or connection parameters a
     *                              subclass might need.
     *
     * @return mixed    The newly created concrete Genie_Driver instance, or
     *                  false on an error.
     */
    function &factory($wishlist, $driver = null, $params = null)
    {
        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        $driver = basename($driver);

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        require_once dirname(__FILE__) . '/Driver/' . $driver . '.php';
        $class = 'Genie_Driver_' . $driver;
        if (class_exists($class)) {
            return $ret = &new $class($wishlist, $params);
        } else {
            return false;
        }
    }

    /**
     * Attempts to return a reference to a concrete Genie_Driver instance
     * based on $driver.  It wil only create a new instance if no
     * Genie_Driver instance with the same parameters currently exists.
     *
     * This should be used if multiple storage sources are required.
     *
     * This method must be invoked as: $var = &Genie_Driver::singleton()
     *
     * @param string    $wishlist   The wishlist to load.
     *
     * @param string    $driver     The type of concrete Genie_Driver subclass
     *                              to return.  The is based on the storage
     *                              driver ($driver).  The code is dynamically
     *                              included.
     *
     * @param array     $params     (optional) A hash containing any additional
     *                              configuration or connection parameters a
     *                              subclass might need.
     *
     * @return mixed    The created concrete Genie_Driver instance, or false
     *                  on error.
     */
    function &singleton($wishlist, $driver = null, $params = null)
    {
        static $instances;

        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        if (!isset($instances)) {
            $instances = array();
        }

        $signature = serialize(array($wishlist, $driver, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Genie_Driver::factory($wishlist, $driver, $params);
        }

        return $instances[$signature];
    }

    /**
     * Get a globally unique ID for an item.
     *
     * @param integer $itemId  The item id.
     *
     * @return string  A GUID referring to $itemId.
     */
    function getGUID($itemId)
    {
        return 'genie:' . $this->_wishlist . ':' . $itemId;
    }

}
