<?php

// Actions
/** @const GENIE_ADD_ITEM Add a new item.           */ define('GENIE_ADD_ITEM', 101);
/** @const GENIE_DELETE_ITEMS Delete items.         */ define('GENIE_DELETE_ITEMS', 102);
/** @const GENIE_MODIFY_ITEM Modify a item.         */ define('GENIE_MODIFY_ITEM', 103);
/** @const GENIE_SAVE_ITEM Save a item to storage.  */ define('GENIE_SAVE_ITEM', 104);
/** @const GENIE_SEARCH_ITEMS Search the item list. */ define('GENIE_SEARCH_ITEMS', 105);
/** @const GENIE_PURCHASE_ITEMS Purchase items.     */ define('GENIE_PURCHASE_ITEMS', 106);
/** @const GENIE_SET_PRIORITY Set item priorities.  */ define('GENIE_SET_PRIORITY', 107);

// Sort types
/** @const GENIE_SORT_NAME Sort by item name.           */ define('GENIE_SORT_NAME', 0);
/** @const GENIE_SORT_PRICE Sort by price.              */ define('GENIE_SORT_PRICE', 1);
/** @const GENIE_SORT_PURCHASED Sort by purchased?.     */ define('GENIE_SORT_PURCHASED', 3);
/** @const GENIE_SORT_CATEGORY Sort by category.        */ define('GENIE_SORT_CATEGORY', 4);
/** @const GENIE_SORT_WISHLIST Sort by wishlist.        */ define('GENIE_SORT_WISHLIST', 5);
/** @const GENIE_SORT_QUANTITY Sort by quantity.        */ define('GENIE_SORT_QUANTITY', 6);
/** @const GENIE_SORT_ASCEND Sort in ascending order.   */ define('GENIE_SORT_ASCEND', 0);
/** @const GENIE_SORT_DESCEND Sort in descending order. */ define('GENIE_SORT_DESCEND', 1);

/**
 * Array for mapping the purchased states to their string representations.
 * @var array $purchased_map
 */
$GLOBALS['purchased_map'] = array(
    0 => _("No"),
    1 => _("Yes"));

/**
 * Genie Base Class.
 *
 * $Horde: genie/lib/Genie.php,v 1.36 2004/12/20 15:44:27 jan Exp $
 *
 * @author Chuck Hagenbuch <chuck@horde.org>
 * @version $Revision: 1.36 $
 * @package Genie
 */
class Genie {

    /**
     * Retrieves the current user's wishlist from storage.
     * This function will also sort the resulting list, if requested.
     *
     * @param object Genie_Driver  $storage   The current storage object.
     * @param constant $sortby   (optional) The field by which to sort.
     *                           (GENIE_SORT_PRIORITY, GENIE_SORT_NAME
     *                            GENIE_SORT_DUE, GENIE_SORT_COMPLETION)
     * @param constant $sortdir  (optional) The direction by which to sort.
     *                           (GENIE_SORT_ASC, GENIE_SORT_DESC)
     *
     * @return array        Returns a list of the requested items.
     *
     * @see Genie_Driver::listItems()
     */
    function listItems($sortby = GENIE_SORT_PRIORITY,
                       $sortdir = GENIE_SORT_ASCEND)
    {
        global $prefs, $conf;
        $items = array();

        /* Sorting criteria for the wishlist. */
        $sort_functions = array(
            GENIE_SORT_NAME => 'ByName',
            GENIE_SORT_PRICE => 'ByPrice',
            GENIE_SORT_PURCHASED => 'ByPurchased',
            GENIE_SORT_CATEGORY => 'ByCategory',
            GENIE_SORT_WISHLIST => 'ByWishlist'
        );

        foreach ($GLOBALS['display_wishlists'] as $wishlist) {
            /* Create a Genie storage instance. */
            $storage = &Genie_Driver::singleton($wishlist);
            $storage->retrieve();

            /* Retrieve the wishlist from storage. */
            $newitems = $storage->listItems();

            /* Filter purchased/unpurchased items if the user doesn't
             * want them shown. */
            if ($prefs->getValue('show_purchased') == 0) {
                foreach ($newitems as $itemId => $item) {
                    if (!empty($item['purchased'])) {
                        unset($newitems[$itemId]);
                    }
                }
            } elseif ($prefs->getValue('show_purchased') == 2) {
                foreach ($newitems as $itemId => $item) {
                    if (empty($item['purchased'])) {
                        unset($newitems[$itemId]);
                    }
                }
            }

            $items = array_merge($items, $newitems);
        }

        // Sort the array if we have a sort function defined for this
        // field.
        if (array_key_exists($sortby, $sort_functions)) {
            $prefix = ($sortdir == GENIE_SORT_DESCEND) ? '_rsort' : '_sort';
            uasort($items, array('Genie', $prefix . $sort_functions[$sortby]));
        }

        return $items;
    }

    function getItem($wishlist, $item)
    {
        $storage = &Genie_Driver::singleton($wishlist);
        return $storage->get($item);
    }

    /**
     * List all wishlists a user has access to.
     *
     * @param optional boolean $owneronly  Only return wishlists that this
     *                                     user owns? Defaults to false.
     * @param optional integer $permission The permission to filter notepads by.
     *
     * @return array  The wishlists.
     */
    function listWishlists($owneronly = false, $permission = PERMS_SHOW)
    {
        $wishlists = $GLOBALS['genie_shares']->listShares(Auth::getAuth(), $permission, $owneronly ? Auth::getAuth() : null);
        if (is_a($wishlists, 'PEAR_Error')) {
            Horde::logMessage($wishlists, __FILE__, __LINE__, PEAR_LOG_ERR);
            return array();
        }

        return $wishlists;
    }

    /**
     * Get the default wishlist for the current user at the specified
     * permissions level.
     */
    function getDefaultWishlist($permission = PERMS_SHOW)
    {
        global $prefs;

        $default_wishlist = $prefs->getValue('default_wishlist');
        $wishlists = Genie::listWishlists(false, $permission);

        if (isset($wishlists[$default_wishlist])) {
            return $default_wishlist;
        } elseif ($prefs->isLocked('default_wishlist')) {
            return '';
        } elseif (count($wishlists)) {
            return key($wishlists);
        }

        return false;
    }

    function listProdIdTypes()
    {
        return array('other' => _("Other"));
    }

    /**
     * Builds the HTML for a item purchased state widget.
     *
     * @param string  $name         The name of the widget.
     * @param integer $selected     (optional) The default purchased state.
     *
     * @return string       The HTML <select> widget.
     */
    function buildPurchasedWidget($name, $selected = 0)
    {
        global $purchased_map;

        $html = "<select name=\"$name\">";

        foreach ($purchased_map as $purchased => $name) {
            $html .= "<option value=\"$purchased\"";
            $html .= ($purchased == $selected) ? ' selected="selected">' : '>';
            $html .= $name . '</option>';
        }
        $html .= "</select>\n";

        return $html;
    }

    /**
     * Builds the HTML for a item prodid_type state widget.
     *
     * @param string  $name         The name of the widget.
     * @param integer $selected     (optional) The default prodid_type state.
     *
     * @return string       The HTML <select> widget.
     */
    function buildProdIdTypeWidget($name, $selected = 0)
    {
        $html = "<select name=\"$name\">";

        foreach (Genie::listProdIdTypes() as $type => $name) {
            $html .= "<option value=\"$type\"";
            $html .= ($type == $selected) ? ' selected="selected">' : '>';
            $html .= $name . '</option>';
        }
        $html .= "</select>\n";

        return $html;
    }

    /**
     * Returns the string representation of the given purchased status.
     *
     * @param int $purchased    The purchased value.
     *
     * @return string       The string representation of $purchased.
     */
    function formatPurchased($purchased)
    {
        global $purchased_map;

        if (isset($purchased_map[$purchased])) {
            return $purchased_map[$purchased];
        }

        return _("Unknown");
    }

    /**
     * Returns the string representation of the given prodid_type.
     *
     * @param int $prodid_type   The prodid_type value.
     *
     * @return string       The string representation of $prodid_type.
     */
    function formatProdIdType($prodid_type)
    {
        $types = Genie::listProdIdTypes();
        if (isset($types[$prodid_type])) {
            return $types[$prodid_type];
        }

        return _("Other");
    }

    /**
     * Build Genie's list of menu items.
     *
     * @access public
     */
    function getMenu($returnType = 'object')
    {
        global $conf, $registry, $browser, $print_link;

        require_once 'Horde/Menu.php';

        $menu = &new Menu(HORDE_MENU_MASK_ALL);
        $menu->add(Horde::applicationUrl('list.php'), _("Wishlist"), 'genie.png');

        if (Genie::getDefaultWishlist(PERMS_EDIT)) {
            $menu->add(Horde::applicationUrl(Util::addParameter('item.php', 'actionID', GENIE_ADD_ITEM)), _("New Item"), 'add.png');
        }
        $menu->add(Horde::applicationUrl('search.php'), _("Search"), 'search.png', $registry->getImageDir('horde'));

        if (Auth::getAuth()) {
            $menu->add(Horde::applicationUrl('wishlists.php'), _("_My Wishlists"), 'wishlists.png');
        }

        /* Print. */
        if ($conf['menu']['print'] && isset($print_link) && $browser->hasFeature('javascript')) {
            $menu->add("javascript:popup('$print_link'); return false;", _("Print"), 'print.png', $registry->getImageDir('horde'));
        }

        if ($returnType == 'object') {
            return $menu;
        } else {
            return $menu->render();
        }
    }

    function getUserEmail($user = null)
    {
        static $results;

        if (is_null($user)) {
            $user = Auth::getAuth();
        } elseif (empty($user)) {
            return '';
        }

        if (!isset($results[$user])) {
            require_once 'Horde/Identity.php';
            $identity = &Identity::singleton('none', $user);

            $results[$user] = $identity->getValue('from_addr');
        }

        return $results[$user];
    }

    /**
     * Comparison function for sorting items by name.
     *
     * @param array $a  Item one.
     * @param array $b  Item two.
     *
     * @return integer  1 if item one is greater, -1 if item two is greater; 0 if they are equal.
     */
    function _sortByName($a, $b)
    {
        return strcmp(preg_replace('|\W|', '', $a['name']),
                      preg_replace('|\W|', '', $b['name']));
    }

    /**
     * Comparison function for reverse sorting items by name.
     *
     * @param array $a  Item one.
     * @param array $b  Item two.
     *
     * @return integer  -1 if item one is greater, 1 if item two is greater; 0 if they are equal.
     */
    function _rsortByName($a, $b)
    {
        return strcmp(preg_replace('|\W|', '', $b['name']),
                      preg_replace('|\W|', '', $a['name']));
    }

    /**
     * Comparison function for sorting items by category.
     *
     * @param array $a  Item one.
     * @param array $b  Item two.
     *
     * @return integer  1 if item one is greater, -1 if item two is greater; 0 if they are equal.
     */
    function _sortByCategory($a, $b)
    {
        return strcasecmp($a['category'], $b['category']);
    }

    /**
     * Comparison function for reverse sorting items by category.
     *
     * @param array $a  Item one.
     * @param array $b  Item two.
     *
     * @return integer  -1 if item one is greater, 1 if item two is greater; 0 if they are equal.
     */
    function _rsortByCategory($a, $b)
    {
        return strcasecmp($b['category'], $a['category']);
    }

    /**
     * Comparison function for sorting items by price.
     *
     * @param array $a  Item one.
     * @param array $b  Item two.
     *
     * @return integer  1 if item one is greater, -1 if item two is greater; 0 if they are equal.
     */
    function _sortByPrice($a, $b)
    {
        $ap = (float)preg_replace('|[^.\w]|', '', $a['price']);
        $bp = (float)preg_replace('|[^.\w]|', '', $b['price']);
        if ($ap == $bp) return 0;
        return ($ap > $bp) ? -1 : 1;
    }

    /**
     * Comparison function for reverse sorting items by price.
     *
     * @param array $a  Item one.
     * @param array $b  Item two.
     *
     * @return integer  -1 if item one is greater, 1 if item two is greater; 0 if they are equal.
     */
    function _rsortByPrice($a, $b)
    {
        $ap = (float)preg_replace('|[^.\w]|', '', $a['price']);
        $bp = (float)preg_replace('|[^.\w]|', '', $b['price']);
        if ($ap == $bp) return 0;
        return ($ap < $bp) ? -1 : 1;
    }

    /**
     * Comparison function for sorting items by purchased status.
     *
     * @param array $a  Item one.
     * @param array $b  Item two.
     *
     * @return integer  1 if item one is greater, -1 if item two is greater; 0 if they are equal.
     */
    function _sortByPurchased($a, $b)
    {
        if ($a['purchased'] == $b['purchased']) return 0;
        return ($a['purchased'] > $b['purchased']) ? -1 : 1;
    }

    /**
     * Comparison function for reverse sorting items by purchased status.
     *
     * @param array $a  Item one.
     * @param array $b  Item two.
     *
     * @return integer  -1 if item one is greater, 1 if item two is greater; 0 if they are equal.
     */
    function _rsortByPurchased($a, $b)
    {
        if ($a['purchased'] == $b['purchased']) return 0;
        return ($a['purchased'] < $b['purchased']) ? -1 : 1;
    }

    /**
     * Comparison function for sorting items by wishlist.
     *
     * @param array $a  Item one.
     * @param array $b  Item two.
     *
     * @return integer  1 if item one is greater, -1 if item two is greater; 0 if they are equal.
     */
    function _sortByWishlist($a, $b)
    {
        $ashare = $GLOBALS['genie_shares']->getShare($a['wishlist_id']);
        $bshare = $GLOBALS['genie_shares']->getShare($b['wishlist_id']);

        $awishlist = $a['wishlist_id'];
        $bwishlist = $b['wishlist_id'];

        if (!is_a($ashare, 'PEAR_Error') && $awishlist != $ashare->get('owner')) {
            $awishlist = $ashare->get('name');
        }
        if (!is_a($bshare, 'PEAR_Error') && $bwishlist != $bshare->get('owner')) {
            $bwishlist = $bshare->get('name');
        }

        return strcasecmp($awishlist, $bwishlist);
    }

    /**
     * Comparison function for reverse sorting items by wishlist.
     *
     * @param array $a  Item one.
     * @param array $b  Item two.
     *
     * @return integer  -1 if item one is greater, 1 if item two is greater; 0 if they are equal.
     */
    function _rsortByWishlist($a, $b)
    {
        $ashare = $GLOBALS['genie_shares']->getShare($a['wishlist_id']);
        $bshare = $GLOBALS['genie_shares']->getShare($b['wishlist_id']);

        $awishlist = $a['wishlist_id'];
        $bwishlist = $b['wishlist_id'];

        if (!is_a($ashare, 'PEAR_Error') && $awishlist != $ashare->get('owner')) {
            $awishlist = $ashare->get('name');
        }
        if (!is_a($bshare, 'PEAR_Error') && $bwishlist != $bshare->get('owner')) {
            $bwishlist = $bshare->get('name');
        }

        return strcasecmp($bwishlist, $awishlist);
    }

}
